@@ -12,7 +12,7 @@ mode for now.
 Quick start
 -----------
 
-Print the routing infotmation in the CSV format::
+Print the routing information in the CSV format::
 
     with NDB() as ndb:
         for record in ndb.routes.summary(format='csv'):
@@ -52,14 +52,6 @@ Change an interface property::
         # ---> <---  the commit() is called authomatically by
         #            the context manager's __exit__()
 
-
-Key NDB features:
-    * Asynchronously updated database of RTNL objects
-    * Data integrity
-    * Multiple data sources -- local, netns, remote
-    * Fault tolerance and memory consumtion limits
-    * Transactions
-
 '''
 import gc
 import sys
@@ -79,11 +71,13 @@ from pyroute2 import config
 from pyroute2 import cli
 from pyroute2.common import basestring
 from pyroute2.ndb import schema
-from pyroute2.ndb.events import (SyncStart,
-                                 MarkFailed,
-                                 DBMExitException,
+from pyroute2.ndb.events import (DBMExitException,
                                  ShutdownException,
                                  InvalidateHandlerException)
+from pyroute2.ndb.messages import (cmsg,
+                                   cmsg_event,
+                                   cmsg_failed,
+                                   cmsg_sstart)
 from pyroute2.ndb.source import Source
 from pyroute2.ndb.objects.interface import Interface
 from pyroute2.ndb.objects.address import Address
@@ -162,6 +156,13 @@ class View(dict):
     def __exit__(self, exc_type, exc_value, traceback):
         pass
 
+    @property
+    def context(self):
+        if self.chain is not None:
+            return self.chain.context
+        else:
+            return {}
+
     def constraint(self, key, value):
         if value is None:
             self.constraints.pop(key)
@@ -184,7 +185,9 @@ class View(dict):
 
     @cli.change_pointer
     def create(self, *argspec, **kwspec):
-        spec = self.classes[self.table].adjust_spec(kwspec or argspec[0])
+        spec = (self
+                .classes[self.table]
+                .adjust_spec(kwspec or argspec[0], self.context))
         if self.chain:
             spec['ndb_chain'] = self.chain
         spec['create'] = True
@@ -283,7 +286,7 @@ class View(dict):
     def __getitem__(self, key, table=None):
 
         iclass = self.classes[table or self.table]
-        key = iclass.adjust_spec(key)
+        key = iclass.adjust_spec(key, self.context)
         if self.match_src:
             match_src = [x for x in self.match_src]
             match_pairs = dict(self.match_pairs)
@@ -297,13 +300,16 @@ class View(dict):
         ret = iclass(self,
                      key,
                      match_src=match_src,
-                     match_pairs=match_pairs)
+                     match_pairs=match_pairs,
+                     load=False)
 
         # rtnl_object.key() returns a dcitionary that can not
         # be used as a cache key. Create here a tuple from it.
         # The key order guaranteed by the dictionary.
         cache_key = tuple(ret.key.items())
 
+        rtime = time.time()
+
         # Iterate all the cache to remove unused and clean
         # (without any started transaction) objects.
         for ckey in tuple(self.cache):
@@ -314,26 +320,27 @@ class View(dict):
             # The number of referrers must be > 1, the first
             # one is the cache itself
             rcount = len(gc.get_referrers(self.cache[ckey]))
+            # Remove only expired items
+            expired = (rtime - self.cache[ckey].atime) > config.cache_expire
             # The number of changed rtnl_object fields must
             # be 0 which means that no transaction is started
-            ccount = len(self.cache[ckey].changed)
-            if rcount == 1 and ccount == 0:
+            if rcount == 1 and self.cache[ckey].clean and expired:
                 self.log.debug('cache del %s' % (ckey, ))
-                del self.cache[ckey]
+                self.cache.pop(ckey, None)
 
-        # Cache only existing objects
-        if ret.state == 'system':
-            if cache_key in self.cache:
-                self.log.debug('cache hit %s' % (cache_key, ))
-                # Explicitly get rid of the created object
-                del ret
-                # The object from the cache has already
-                # registered callbacks, simply return it
-                ret = self.cache[cache_key]
-                return ret
-            else:
+        if cache_key in self.cache:
+            self.log.debug('cache hit %s' % (cache_key, ))
+            # Explicitly get rid of the created object
+            del ret
+            # The object from the cache has already
+            # registered callbacks, simply return it
+            ret = self.cache[cache_key]
+            ret.atime = rtime
+            return ret
+        else:
+            # Cache only existing objects
+            if ret.load_sql():
                 self.log.debug('cache add %s' % (cache_key, ))
-                # Otherwise create a cache entry
                 self.cache[cache_key] = ret
 
         ret.register()
@@ -786,7 +793,7 @@ class NDB(object):
                         'kind': 'local',
                         'nlm_generator': 1}]
             if sys.platform.startswith('linux'):
-                sources.append({'target': 'localhost/netns',
+                sources.append({'target': 'nsmanager',
                                 'kind': 'nsmanager'})
         elif not isinstance(sources, (list, tuple)):
             raise ValueError('sources format not supported')
@@ -812,7 +819,7 @@ class NDB(object):
         self.routes = View(self, 'routes')
         self.neighbours = View(self, 'neighbours')
         self.rules = View(self, 'rules')
-        self.netns = View(self, 'netns', default_target='localhost/netns')
+        self.netns = View(self, 'netns', default_target='nsmanager')
         self.query = Query(self.schema)
 
     def _get_view(self, name, match_src=None, match_pairs=None, chain=None):
@@ -850,14 +857,14 @@ class NDB(object):
                     pass
             # shutdown the _dbm_thread
             self._event_queue.shutdown()
-            self._event_queue.bypass(('localhost', (ShutdownException(), )))
+            self._event_queue.bypass((cmsg(None, ShutdownException()), ))
             self._dbm_thread.join()
 
     def __dbm__(self):
 
         def default_handler(target, event):
-            if isinstance(event, Exception):
-                raise event
+            if isinstance(getattr(event, 'payload', None), Exception):
+                raise event.payload
             log.warning('unsupported event ignored: %s' % type(event))
 
         def check_sources_started(self, _locals, target, event):
@@ -868,12 +875,12 @@ class NDB(object):
         _locals = {'countdown': len(self._nl)}
 
         # init the events map
-        event_map = {type(self._dbm_ready): [lambda t, x: x.set()],
-                     MarkFailed: [lambda t, x: (self
-                                                .schema
-                                                .mark(t, 1))],
-                     SyncStart: [partial(check_sources_started,
-                                         self, _locals)]}
+        event_map = {cmsg_event: [lambda t, x: x.payload.set()],
+                     cmsg_failed: [lambda t, x: (self
+                                                 .schema
+                                                 .mark(t, 1))],
+                     cmsg_sstart: [partial(check_sources_started,
+                                           self, _locals)]}
         self._event_map = event_map
 
         event_queue = self._event_queue
@@ -899,7 +906,8 @@ class NDB(object):
 
         stop = False
         while not stop:
-            target, events = event_queue.get()
+            events = event_queue.get()
+            log.error(events)
             try:
                 if events is None:
                     continue
@@ -910,6 +918,7 @@ class NDB(object):
                                              [default_handler, ])
                     for handler in tuple(handlers):
                         try:
+                            target = event['header']['target']
                             handler(target, event)
                         except InvalidateHandlerException:
                             try:
