@@ -28,10 +28,10 @@ E.g.::
     with NDB(log="stderr") as ndb:
         (ndb
          .interfaces
-         .add(ifname="vlan1108",
-              kind="vlan",
-              link="eth0",
-              vlan_id=1108)
+         .create(ifname="vlan1108",
+                 kind="vlan",
+                 link="eth0",
+                 vlan_id=1108)
          .commit())
 
 Slightly simplifying, if a network object doesn't exist, NDB will run
@@ -49,6 +49,7 @@ NDB objects are grouped into "views":
     * neighbours
     * rules
     * netns
+    * ...
 
 Views are dictionary-like objects that accept strings or dict selectors::
 
@@ -105,6 +106,7 @@ class RTNL_Object(dict):
     table_alias = ''
 
     key_extra_fields = []
+    hidden_fields = []
     fields_cmp = {}
 
     schema = None
@@ -193,7 +195,8 @@ class RTNL_Object(dict):
                  iclass,
                  ctxid=None,
                  match_src=None,
-                 match_pairs=None):
+                 match_pairs=None,
+                 load=True):
         self.view = view
         self.ndb = view.ndb
         self.sources = view.ndb.sources
@@ -205,6 +208,7 @@ class RTNL_Object(dict):
         self.iclass = iclass
         self.utable = self.utable or self.table
         self.errors = []
+        self.atime = time.time()
         self.log = self.ndb.log.channel('rtnl_object')
         self.log.debug('init')
         self.state = State()
@@ -225,25 +229,37 @@ class RTNL_Object(dict):
             self.chain = None
             create = False
         ckey = self.complete_key(key)
-        if create and ckey is not None:
-            raise KeyError('object exists')
-        elif not create and ckey is None:
-            raise KeyError('object does not exists')
-        elif create:
+        if create:
+            if ckey is not None:
+                raise KeyError('object exists')
             for name in key:
                 self[name] = key[name]
             # FIXME -- merge with complete_key()
             if 'target' not in self:
                 self.load_value('target', self.view.default_target)
-        elif ctxid is None:
-            self.key = ckey
-            self.load_sql()
         else:
+            if ckey is None:
+                raise KeyError('object does not exists')
             self.key = ckey
-            self.load_sql(table=self.table)
+            if load:
+                if ctxid is None:
+                    self.load_sql()
+                else:
+                    self.load_sql(table=self.table)
+
+    def mark_tflags(self, mark):
+        pass
+
+    def keys(self):
+        return filter(lambda x: x not in self.hidden_fields,
+                      dict.keys(self))
+
+    def items(self):
+        return filter(lambda x: x[0] not in self.hidden_fields,
+                      dict.items(self))
 
     @classmethod
-    def adjust_spec(cls, spec):
+    def adjust_spec(cls, spec, context):
         return spec
 
     @classmethod
@@ -254,6 +270,10 @@ class RTNL_Object(dict):
     def name2nla(self, name):
         return self.msg_class.name2nla(name)
 
+    @property
+    def context(self):
+        return {'target': self.get('target', 'localhost')}
+
     def __enter__(self):
         return self
 
@@ -456,14 +476,21 @@ class RTNL_Object(dict):
             link.rollback(snapshot=snp)
         return self
 
+    def clear(self):
+        pass
+
+    @property
+    def clean(self):
+        return self.state == 'system' and \
+            not self.changed and \
+            not self._apply_script
+
     def commit(self):
         '''
         Try to commit the pending changes. If the commit fails,
         automatically revert the state.
         '''
-        if self.state == 'system' and \
-                not self.changed and \
-                not self._apply_script:
+        if self.clean:
             return self
 
         if self.chain:
