@@ -90,51 +90,42 @@ There are also some useful views, that join `ifinfo` tables with
 '''
 import sys
 import time
-import uuid
-import struct
 import random
 import sqlite3
 import threading
 import traceback
 from functools import partial
 from collections import OrderedDict
-from socket import (AF_INET,
-                    inet_pton)
 from pyroute2 import config
-from pyroute2.config import AF_BRIDGE
 from pyroute2.common import uuid32
-from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
-from pyroute2.netlink.rtnl.ifaddrmsg import ifaddrmsg
-from pyroute2.netlink.rtnl.ndmsg import ndmsg
-from pyroute2.netlink.rtnl.rtmsg import rtmsg
-from pyroute2.netlink.rtnl.rtmsg import nh
-from pyroute2.netlink.rtnl.fibmsg import fibmsg
-from pyroute2.netlink.rtnl.p2pmsg import p2pmsg
-from pyroute2.netlink.rtnl.nsinfmsg import nsinfmsg
+from pyroute2.common import basestring
+# plugins
+from pyroute2.ndb.objects import address
+from pyroute2.ndb.objects import interface
+from pyroute2.ndb.objects import neighbour
+from pyroute2.ndb.objects import netns
+from pyroute2.ndb.objects import route
+from pyroute2.ndb.objects import rule
 
-# rtnl objects
-from pyroute2.ndb.objects.route import Route
-from pyroute2.ndb.objects.route import NextHop
-from pyroute2.ndb.objects.address import Address
-
-# events
-from pyroute2.ndb.events import SchemaGenericRequest
+#
+from pyroute2.ndb.messages import cmsg
 
 try:
     import queue
 except ImportError:
     import Queue as queue
 
+#
+# the order is important
+#
+plugins = [interface,
+           address,
+           neighbour,
+           route,
+           netns,
+           rule]
+
 MAX_ATTEMPTS = 5
-ifinfo_names = ('bridge',
-                'bond',
-                'vlan',
-                'vxlan',
-                'gre',
-                'vrf',
-                'vti',
-                'vti6')
-supported_ifinfo = {x: ifinfmsg.ifinfo.data_map[x] for x in ifinfo_names}
 
 
 def publish(method):
@@ -152,8 +143,12 @@ def publish(method):
     #
     # this class will be used to map the requests
     #
-    class SchemaRequest(SchemaGenericRequest):
-        pass
+    class cmsg_req(cmsg):
+        def __init__(self, response, *argv, **kwarg):
+            self['header'] = {'target': None}
+            self.response = response
+            self.argv = argv
+            self.kwarg = kwarg
 
     #
     # this method will replace the source one
@@ -167,8 +162,8 @@ def publish(method):
             # another thread, run via message bus
             self._allow_read.wait()
             response = queue.Queue(maxsize=4096)
-            request = SchemaRequest(response, *argv, **kwarg)
-            self.ndb._event_queue.put((None, (request, )))
+            request = cmsg_req(response, *argv, **kwarg)
+            self.ndb._event_queue.put((request, ))
             while True:
                 item = response.get()
                 if isinstance(item, StopIteration):
@@ -181,7 +176,7 @@ def publish(method):
     #
     # announce the function so it will be published
     #
-    _do_dispatch.publish = (SchemaRequest, _do_local)
+    _do_dispatch.publish = (cmsg_req, _do_local)
 
     return _do_dispatch
 
@@ -203,8 +198,12 @@ def publish_exec(method):
     #
     # this class will be used to map the requests
     #
-    class SchemaRequest(SchemaGenericRequest):
-        pass
+    class cmsg_req(cmsg):
+        def __init__(self, response, *argv, **kwarg):
+            self['header'] = {'target': None}
+            self.response = response
+            self.argv = argv
+            self.kwarg = kwarg
 
     #
     # this method will replace the source one
@@ -216,8 +215,8 @@ def publish_exec(method):
         else:
             # another thread, run via message bus
             response = queue.Queue(maxsize=1)
-            request = SchemaRequest(response, *argv, **kwarg)
-            self.ndb._event_queue.put((None, (request, )))
+            request = cmsg_req(response, *argv, **kwarg)
+            self.ndb._event_queue.put((request, ))
             ret = response.get()
             if isinstance(ret, Exception):
                 raise ret
@@ -227,7 +226,7 @@ def publish_exec(method):
     #
     # announce the function so it will be published
     #
-    _do_dispatch.publish = (SchemaRequest, _do_local)
+    _do_dispatch.publish = (cmsg_req, _do_local)
 
     return _do_dispatch
 
@@ -241,157 +240,14 @@ class DBSchema(object):
     snapshots = None  # <table_name>: <obj_weakref>
 
     spec = OrderedDict()
-    # main tables
-    spec['interfaces'] = OrderedDict(ifinfmsg.sql_schema())
-    spec['addresses'] = OrderedDict(ifaddrmsg.sql_schema())
-    spec['neighbours'] = OrderedDict(ndmsg.sql_schema())
-    spec['routes'] = OrderedDict(rtmsg.sql_schema() +
-                                 [(('route_id', ), 'TEXT UNIQUE'),
-                                  (('gc_mark', ), 'INTEGER')])
-    spec['nh'] = OrderedDict(nh.sql_schema() +
-                             [(('route_id', ), 'TEXT'),
-                              (('nh_id', ), 'INTEGER')])
-    spec['rules'] = OrderedDict(fibmsg.sql_schema())
-    spec['netns'] = OrderedDict(nsinfmsg.sql_schema())
-    # additional tables
-    spec['p2p'] = OrderedDict(p2pmsg.sql_schema())
-
-    classes = {'interfaces': ifinfmsg,
-               'addresses': ifaddrmsg,
-               'neighbours': ndmsg,
-               'routes': rtmsg,
-               'nh': nh,
-               'rules': fibmsg,
-               'netns': nsinfmsg,
-               'p2p': p2pmsg}
-
+    classes = {}
     #
     # OBS: field names MUST go in the same order as in the spec,
     # that's for the load_netlink() to work correctly -- it uses
     # one loop to fetch both index and row values
     #
-    indices = {'interfaces': ('index', ),
-               'p2p': ('index', ),
-               'addresses': ('family',
-                             'prefixlen',
-                             'index',
-                             'IFA_ADDRESS',
-                             'IFA_LOCAL'),
-               'neighbours': ('ifindex',
-                              'NDA_LLADDR'),
-               'routes': ('family',
-                          'dst_len',
-                          'tos',
-                          'RTA_DST',
-                          'RTA_PRIORITY',
-                          'RTA_TABLE'),
-               'nh': ('route_id',
-                      'nh_id'),
-               'netns': ('NSINFO_PATH', ),
-               'rules': ('family',
-                         'dst_len',
-                         'src_len',
-                         'tos',
-                         'action',
-                         'flags',
-                         'FRA_DST',
-                         'FRA_SRC',
-                         'FRA_IIFNAME',
-                         'FRA_GOTO',
-                         'FRA_PRIORITY',
-                         'FRA_FWMARK',
-                         'FRA_FLOW',
-                         'FRA_TUN_ID',
-                         'FRA_SUPPRESS_IFGROUP',
-                         'FRA_SUPPRESS_PREFIXLEN',
-                         'FRA_TABLE',
-                         'FRA_FWMASK',
-                         'FRA_OIFNAME',
-                         'FRA_L3MDEV',
-                         'FRA_UID_RANGE',
-                         'FRA_PROTOCOL',
-                         'FRA_IP_PROTO',
-                         'FRA_SPORT_RANGE',
-                         'FRA_DPORT_RANGE')}
-
-    foreign_keys = {'addresses': [{'fields': ('f_target',
-                                              'f_tflags',
-                                              'f_index'),
-                                   'parent_fields': ('f_target',
-                                                     'f_tflags',
-                                                     'f_index'),
-                                   'parent': 'interfaces'}],
-                    'neighbours': [{'fields': ('f_target',
-                                               'f_tflags',
-                                               'f_ifindex'),
-                                    'parent_fields': ('f_target',
-                                                      'f_tflags',
-                                                      'f_index'),
-                                    'parent': 'interfaces'}],
-                    'routes': [{'fields': ('f_target',
-                                           'f_tflags',
-                                           'f_RTA_OIF'),
-                                'parent_fields': ('f_target',
-                                                  'f_tflags',
-                                                  'f_index'),
-                                'parent': 'interfaces'},
-                               {'fields': ('f_target',
-                                           'f_tflags',
-                                           'f_RTA_IIF'),
-                                'parent_fields': ('f_target',
-                                                  'f_tflags',
-                                                  'f_index'),
-                                'parent': 'interfaces'}],
-                    #
-                    # man kan not use f_tflags together with f_route_id
-                    # 'cause it breaks ON UPDATE CASCADE for interfaces
-                    #
-                    'nh': [{'fields': ('f_route_id', ),
-                            'parent_fields': ('f_route_id', ),
-                            'parent': 'routes'},
-                           {'fields': ('f_target',
-                                       'f_tflags',
-                                       'f_oif'),
-                            'parent_fields': ('f_target',
-                                              'f_tflags',
-                                              'f_index'),
-                            'parent': 'interfaces'}],
-                    #
-                    # additional tables
-                    #
-                    'p2p': [{'fields': ('f_target',
-                                        'f_tflags',
-                                        'f_index'),
-                             'parent_fields': ('f_target',
-                                               'f_tflags',
-                                               'f_index'),
-                             'parent': 'interfaces'}]}
-
-    #
-    # load supported ifinfo
-    #
-    for (name, data) in supported_ifinfo.items():
-        name = 'ifinfo_%s' % name
-        #
-        # classes
-        #
-        classes[name] = data
-        #
-        # indices
-        #
-        indices[name] = ('index', )
-        #
-        # spec
-        #
-        spec[name] = \
-            OrderedDict(data.sql_schema() + [(('index', ), 'BIGINT')])
-        #
-        # foreign keys
-        #
-        foreign_keys[name] = \
-            [{'fields': ('f_target', 'f_tflags', 'f_index'),
-              'parent_fields': ('f_target', 'f_tflags', 'f_index'),
-              'parent': 'interfaces'}]
+    indices = {}
+    foreign_keys = {}
 
     def __init__(self, ndb, connection, mode, rtnl_log, tid):
         self.ndb = ndb
@@ -410,6 +266,7 @@ class DBSchema(object):
         self.log = ndb.log.channel('schema')
         self.snapshots = {}
         self.key_defaults = {}
+        self.event_map = {}
         self._cursor = None
         self._counter = 0
         self._allow_read = threading.Event()
@@ -447,37 +304,6 @@ class DBSchema(object):
                 self.create_ifinfo_view(table)
 
         #
-        # specific SQL code
-        #
-        if self.mode == 'sqlite3':
-            template = '''
-                       CREATE TRIGGER IF NOT EXISTS {name}
-                       BEFORE UPDATE OF {field} ON {table} FOR EACH ROW
-                           BEGIN
-                               {sql}
-                           END
-                       '''
-        elif self.mode == 'psycopg2':
-            template = '''
-                       CREATE OR REPLACE FUNCTION {name}()
-                       RETURNS trigger AS ${name}$
-                           BEGIN
-                               {sql}
-                               RETURN NEW;
-                           END;
-                       ${name}$ LANGUAGE plpgsql;
-
-                       DROP TRIGGER IF EXISTS {name} ON {table};
-
-                       CREATE TRIGGER {name}
-                       BEFORE UPDATE OF {field} ON {table} FOR EACH ROW
-                       EXECUTE PROCEDURE {name}();
-                       '''
-
-        for cls in (NextHop, Route, Address):
-            self.execute(template.format(**cls.reverse_update))
-            self.connection.commit()
-        #
         # service tables
         #
         self.execute('''
@@ -611,13 +437,14 @@ class DBSchema(object):
             cursor = self._cursor
         else:
             cursor = self.connection.cursor()
-            self._counter = config.db_transaction_limit + 1
+            self._counter = config.db_transaction_limit
         try:
             #
             # FIXME: add logging
             #
             for _ in range(MAX_ATTEMPTS):
                 try:
+                    self._counter += 1
                     cursor.execute(*argv, **kwarg)
                     break
                 except (sqlite3.InterfaceError, sqlite3.OperationalError) as e:
@@ -734,6 +561,7 @@ class DBSchema(object):
         self.connection.commit()
 
     def create_ifinfo_view(self, table, ctxid=None):
+        # FIXME: move to the object module
         iftable = 'interfaces'
 
         req = (('main.f_target', 'main.f_tflags') +
@@ -876,18 +704,11 @@ class DBSchema(object):
         #
         # mark tflags for obj
         #
-        self.execute('''
-                     UPDATE %s SET
-                         f_tflags = %s
-                     WHERE %s
-                     '''
-                     % (obj.utable,
-                        self.plch,
-                        ' AND '.join(conditions)),
-                     [uuid] + values)
+        obj.mark_tflags(uuid)
+
         #
-        # t_flags is used in foreign keys ON UPDATE CASCADE, so all
-        # related records will be marked, now just copy the marked data
+        # f_tflags is used in foreign keys ON UPDATE CASCADE, so all
+        # related records will be marked
         #
         for table in self.spec:
             self.log.debug('create snapshot %s_%s' % (table, ctxid))
@@ -918,15 +739,8 @@ class DBSchema(object):
         #
         # unmark all the data
         #
-        self.execute('''
-                     UPDATE %s SET
-                         f_tflags = %s
-                     WHERE %s
-                     '''
-                     % (obj.utable,
-                        self.plch,
-                        ' AND '.join(conditions)),
-                     [tflags] + values)
+        obj.mark_tflags(tflags)
+
         for table in self.spec:
             self.execute('''
                          UPDATE %s_%s SET f_tflags = %s
@@ -941,14 +755,15 @@ class DBSchema(object):
                     if table.startswith('ifinfo_'):
                         try:
                             self.execute('DROP VIEW %s' % table[7:])
+                            self.connection.commit()
                         except Exception:
                             # GC collision?
-                            self.log.warning('purge_snapshots: %s'
-                                             % traceback.format_exc())
+                            pass
                     if self.mode == 'sqlite3':
                         self.execute('DROP TABLE %s' % table)
                     elif self.mode == 'psycopg2':
                         self.execute('DROP TABLE %s CASCADE' % table)
+                    self.connection.commit()
                     del self.snapshots[table]
                     break
                 except sqlite3.OperationalError:
@@ -982,221 +797,6 @@ class DBSchema(object):
         for record in self.fetch(req, values):
             yield dict(zip(self.compiled[table]['all_names'], record))
 
-    def rtmsg_gc_mark(self, target, event, gc_mark=None):
-        #
-        if gc_mark is None:
-            gc_clause = ' AND f_gc_mark IS NOT NULL'
-        else:
-            gc_clause = ''
-        #
-        # select all routes for that OIF where f_gc_mark is not null
-        #
-        key_fields = ','.join(['f_%s' % x for x
-                               in self.indices['routes']])
-        key_query = ' AND '.join(['f_%s = %s' % (x, self.plch) for x
-                                  in self.indices['routes']])
-        routes = (self
-                  .execute('''
-                           SELECT %s,f_RTA_GATEWAY FROM routes WHERE
-                           f_target = %s AND f_RTA_OIF = %s AND
-                           f_RTA_GATEWAY IS NOT NULL %s
-                           ''' % (key_fields,
-                                  self.plch,
-                                  self.plch,
-                                  gc_clause),
-                           (target, event.get_attr('RTA_OIF')))
-                  .fetchmany())
-        #
-        # get the route's RTA_DST and calculate the network
-        #
-        addr = event.get_attr('RTA_DST')
-        net = struct.unpack('>I', inet_pton(AF_INET, addr))[0] &\
-            (0xffffffff << (32 - event['dst_len']))
-        #
-        # now iterate all the routes from the query above and
-        # mark those with matching RTA_GATEWAY
-        #
-        for route in routes:
-            # get route GW
-            gw = route[-1]
-            gwnet = struct.unpack('>I', inet_pton(AF_INET, gw))[0] & net
-            if gwnet == net:
-                (self
-                 .execute('UPDATE routes SET f_gc_mark = %s '
-                          'WHERE f_target = %s AND %s'
-                          % (self.plch, self.plch, key_query),
-                          (gc_mark, target) + route[:-1]))
-
-    def load_nsinfmsg(self, target, event):
-        #
-        # check if there is corresponding source
-        #
-        netns_path = event.get_attr('NSINFO_PATH')
-        if netns_path is None:
-            self.log.debug('ignore %s %s' % (target, event))
-            return
-        if self.ndb._auto_netns:
-            if netns_path.find('/var/run/docker') > -1:
-                source_name = 'docker/%s' % netns_path.split('/')[-1]
-            else:
-                source_name = 'netns/%s' % netns_path.split('/')[-1]
-            if event['header'].get('type', 0) % 2:
-                if source_name in self.ndb.sources.cache:
-                    self.ndb.sources.remove(source_name, code=108, sync=False)
-            elif source_name not in self.ndb.sources.cache:
-                sync_event = None
-                if self.ndb._dbm_autoload and not self.ndb._dbm_ready.is_set():
-                    sync_event = threading.Event()
-                    self.ndb._dbm_autoload.add(sync_event)
-                    self.log.debug('queued event %s' % sync_event)
-                else:
-                    sync_event = None
-                self.log.debug('starting netns source %s' % source_name)
-                self.ndb.sources.async_add(target=source_name,
-                                           netns=netns_path,
-                                           persistent=False,
-                                           event=sync_event)
-        self.load_netlink('netns', target, event)
-
-    def load_ndmsg(self, target, event):
-        #
-        # ignore events with ifindex == 0
-        #
-        if event['ifindex'] == 0:
-            return
-
-        self.load_netlink('neighbours', target, event)
-
-    def load_ifinfmsg(self, target, event):
-        #
-        # link goes down: flush all related routes
-        #
-        if not event['flags'] & 1:
-            self.execute('DELETE FROM routes WHERE '
-                         'f_target = %s AND '
-                         'f_RTA_OIF = %s OR f_RTA_IIF = %s'
-                         % (self.plch, self.plch, self.plch),
-                         (target, event['index'], event['index']))
-        #
-        # ignore wireless updates
-        #
-        if event.get_attr('IFLA_WIRELESS'):
-            return
-        #
-        # AF_BRIDGE events
-        #
-        if event['family'] == AF_BRIDGE:
-            #
-            # bypass for now
-            #
-            return
-
-        self.load_netlink('interfaces', target, event)
-        #
-        # load ifinfo, if exists
-        #
-        if not event['header'].get('type', 0) % 2:
-            linkinfo = event.get_attr('IFLA_LINKINFO')
-            if linkinfo is not None:
-                iftype = linkinfo.get_attr('IFLA_INFO_KIND')
-                table = 'ifinfo_%s' % iftype
-                if iftype == 'gre':
-                    ifdata = linkinfo.get_attr('IFLA_INFO_DATA')
-                    local = ifdata.get_attr('IFLA_GRE_LOCAL')
-                    remote = ifdata.get_attr('IFLA_GRE_REMOTE')
-                    p2p = p2pmsg()
-                    p2p['index'] = event['index']
-                    p2p['family'] = 2
-                    p2p['attrs'] = [('P2P_LOCAL', local),
-                                    ('P2P_REMOTE', remote)]
-                    self.load_netlink('p2p', target, p2p)
-                elif iftype == 'veth':
-                    link = event.get_attr('IFLA_LINK')
-                    # for veth interfaces, IFLA_LINK points to
-                    # the peer -- but NOT in automatic updates
-                    if (not link) and \
-                            (target in self.ndb.sources.keys()):
-                        self.log.debug('reload veth %s' % event['index'])
-                        update = (self
-                                  .ndb
-                                  .sources[target]
-                                  .api('link', 'get', index=event['index']))
-                        update = tuple(update)[0]
-                        return self.load_netlink('interfaces', target, update)
-
-                if table in self.spec:
-                    ifdata = linkinfo.get_attr('IFLA_INFO_DATA')
-                    ifdata['header'] = {}
-                    ifdata['index'] = event['index']
-                    self.load_netlink(table, target, ifdata)
-
-    def load_rtmsg(self, target, event):
-        mp = event.get_attr('RTA_MULTIPATH')
-
-        # create an mp route
-        if (not event['header']['type'] % 2) and mp:
-            #
-            # create key
-            keys = ['f_target = %s' % self.plch]
-            values = [target]
-            for key in self.indices['routes']:
-                keys.append('f_%s = %s' % (key, self.plch))
-                values.append(event.get(key) or event.get_attr(key))
-            #
-            spec = 'WHERE %s' % ' AND '.join(keys)
-            s_req = 'SELECT f_route_id FROM routes %s' % spec
-            #
-            # get existing route_id
-            for route_id in self.execute(s_req, values).fetchall():
-                #
-                # if exists
-                route_id = route_id[0][0]
-                #
-                # flush all previous MP hops
-                d_req = 'DELETE FROM nh WHERE f_route_id= %s' % self.plch
-                self.execute(d_req, (route_id, ))
-                break
-            else:
-                #
-                # or create a new route_id
-                route_id = str(uuid.uuid4())
-            #
-            # set route_id on the route itself
-            event['route_id'] = route_id
-            self.load_netlink('routes', target, event)
-            for idx in range(len(mp)):
-                mp[idx]['header'] = {}          # for load_netlink()
-                mp[idx]['route_id'] = route_id  # set route_id on NH
-                mp[idx]['nh_id'] = idx          # add NH number
-                self.load_netlink('nh', target, mp[idx], 'routes')
-            #
-            # we're done with an MP-route, just exit
-            return
-        #
-        # manage gc marks on related routes
-        #
-        # only for automatic routes:
-        #   - table 254 (main)
-        #   - proto 2 (kernel)
-        #   - scope 253 (link)
-        elif (event.get_attr('RTA_TABLE') == 254) and \
-                (event['proto'] == 2) and \
-                (event['scope'] == 253) and \
-                (event['family'] == AF_INET):
-            evt = event['header']['type']
-            #
-            # set f_gc_mark = timestamp for "del" events
-            # and clean it for "new" events
-            #
-            self.rtmsg_gc_mark(target, event,
-                               int(time.time()) if (evt % 2) else None)
-            #
-            # continue with load_netlink()
-            #
-        #
-        # ... or work on a regular route
-        self.load_netlink("routes", target, event)
-
     def log_netlink(self, table, target, event, ctable=None):
         #
         # RTNL Logs
@@ -1363,11 +963,47 @@ class DBSchema(object):
 
 
 def init(ndb, connection, mode, rtnl_log, tid):
+    #
+    # first prepare the schema and init the DB
+    #
+    for plugin in plugins:
+        #
+        # 1. spec
+        #
+        for name, spec in plugin.init['specs']:
+            DBSchema.spec[name] = spec
+        #
+        # 2. classes
+        #
+        for name, cls in plugin.init['classes']:
+            DBSchema.classes[name] = cls
+        #
+        # 3. indices
+        #
+        for name, idx in plugin.init['indices']:
+            DBSchema.indices[name] = idx
+        #
+        # 4. foreign keys
+        #
+        for name, kspec in plugin.init['foreign_keys']:
+            DBSchema.foreign_keys[name] = kspec
+
     ret = DBSchema(ndb, connection, mode, rtnl_log, tid)
-    ret.event_map = {ifinfmsg: [ret.load_ifinfmsg],
-                     ifaddrmsg: [partial(ret.load_netlink, 'addresses')],
-                     ndmsg: [ret.load_ndmsg],
-                     rtmsg: [ret.load_rtmsg],
-                     fibmsg: [partial(ret.load_netlink, 'rules')],
-                     nsinfmsg: [ret.load_nsinfmsg]}
+
+    #
+    # init the event mapping
+    #
+    for plugin in plugins:
+        #
+        emap = plugin.init['event_map']
+        #
+        for etype, ehndl in emap.items():
+            handlers = []
+            for h in ehndl:
+                if isinstance(h, basestring):
+                    handlers.append(partial(ret.load_netlink, h))
+                else:
+                    handlers.append(partial(h, ret))
+            ret.event_map[etype] = handlers
+
     return ret
